const https = require('https');
const url = require('url');
const path = require('path');
var zlib = require('zlib');
var fs = require('fs');

const downloadFileAndGunzip = (fileToDownload, filename, destination, dontGunzip) => {
  https
    .get(fileToDownload, (res) => {
      let data = [];
      if (
        res.statusCode > 300 &&
        res.statusCode < 400 &&
        res.headers.location
      ) {
        console.log('Redirected from GitHub release page');
        if (url.parse(res.headers.location).href) {
          const fileToDownloadRedirect = url.parse(res.headers.location).href;
          console.log('Got release location', fileToDownloadRedirect);
          https
            .get(fileToDownloadRedirect, (res2) => {
              const headerDate2 =
                res2.headers && res2.headers.date
                  ? res2.headers.date
                  : 'no response data';
              console.log('Status Code:', res2.statusCode);
              console.log('Date in Response header:', headerDate2);

              res2.on('data', (chunk) => {
                data.push(chunk);
              });

              res2.on('end', () => {
                console.log('Downloaded binary to buffer: ');
                const fileData = Buffer.concat(data);
                // fs.writeFileSync('libpact_ffi-osx-x86_64.dylib.gz', datarrr)
                // 46d70e422c7f49b851de4121f7a15c296171b7e3fd8ec2be4f4e1fa8d374e077 actual
                const filePath = destination ? path.join(destination, filename) : filename
                if (destination) {
                  fs.mkdir(destination, (err) => {
                    if (err) {
                      console.log(
                        'error occurred in creating new directory, this might be okay if download more than 1 file (windows)');
                    }
                  });
                }
                if (dontGunzip){
                  fs.writeFileSync(
                    filePath,
                    fileData
                  );
                }else {
                  zlib.gunzip(fileData, (err, buffer) => {
    
  
                    fs.writeFileSync(
                      filePath,
                      buffer
                    );
                  });
                }
                console.log('written file to:',filePath)
              });
            })
            .on('error', (err) => {
              console.log('Error: ', err.message);
            });
        } else {
          console.log('unable to get path to download');
        }
      }

      const headerDate =
        res.headers && res.headers.date ? res.headers.date : 'no response data';
      console.log('Status Code:', res.statusCode);
      console.log('Date in Response header:', headerDate);
      res.on('end', () => {
        console.log('redirect complete: ');
      });
    })
    .on('error', (err) => {
      console.log('Error: ', err.message);
    });
};

const PACT_FFI_VERSION = '0.4.3';
const PACT_BINARY_LOCATION = `https://github.com/pact-foundation/pact-reference/releases/download/libpact_ffi-v${PACT_FFI_VERSION}/`;
const CHECKSUM_SUFFIX = '.sha256';
const PREFIX = 'lib';

switch (process.platform) {
  case 'win32':
    console.log('downloading FFI for windows')
    downloadFileAndGunzip(
      PACT_BINARY_LOCATION + 'pact_ffi-windows-x86_64.dll.gz',
      'pact_ffi.dll',
      'ffi'
    );
    downloadFileAndGunzip(
      PACT_BINARY_LOCATION + 'pact_ffi-windows-x86_64.dll.lib.gz',
      'pact_ffi.dll.lib',
      'ffi'
    );
    break;
  case 'darwin':
    if (process.arch === 'x64') {
      console.log('downloading FFI for osx x64')
      downloadFileAndGunzip(
        PACT_BINARY_LOCATION + 'libpact_ffi-osx-x86_64.dylib.gz',
        'libpact_ffi.dylib',
        'ffi/osxx64'
      );
    }
    if (process.arch === 'arm64') {
      console.log('downloading FFI for osx arm64')
      downloadFileAndGunzip(
        PACT_BINARY_LOCATION + `libpact_ffi-osx-aarch64-apple-darwin.dylib.gz`,
        'libpact_ffi.dylib',
        'ffi/osxaarch64'
      );
    }
    break;
  case 'linux':
    if (process.arch === 'x64') {
      console.log('downloading FFI for linux x64')
      downloadFileAndGunzip(
        PACT_BINARY_LOCATION + 'libpact_ffi-linux-x86_64.so.gz',
        'libpact_ffi.so',
        'ffi/linuxx64'
      );
    }
    if (process.arch === 'arm64') {
      console.log('downloading FFI for linux arm64')
      downloadFileAndGunzip(
        PACT_BINARY_LOCATION + `libpact_ffi-linux-aarch64.so.gz`,
        'libpact_ffi.so',
        'ffi/linuxaarch64'
      );
    }
    break;

  default:
    break;
}

console.log('downloading FFI header files')
downloadFileAndGunzip(
  PACT_BINARY_LOCATION + `pact.h`,
  'pact.h',
  'ffi',
  true
);
downloadFileAndGunzip(
  PACT_BINARY_LOCATION + `pact-cpp.h`,
  'pact-cpp.h',
  'ffi',
  true
);