"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadChecksums = exports.getBinaryEntry = exports.createConfig = exports.PACT_STANDALONE_VERSION = exports.PactEnvironment = void 0;
const needle = __importStar(require("needle"));
const unzipper = require("unzipper");
const tar = require("tar");
const chalk_1 = __importDefault(require("chalk"));
const path = require("path");
const fs = require("fs");
const urljoin = require("url-join");
const sumchecker = require("sumchecker");
const config = require('libnpmconfig');
class PactEnvironment {
    get cwd() {
        return path.resolve(__dirname, '..');
    }
    isWindows(platform = process.platform) {
        return platform === 'win32';
    }
}
exports.PactEnvironment = PactEnvironment;
const pactEnvironment = new PactEnvironment();
const environmentProxy = process.env['npm_config_https_proxy'] || process.env['npm_config_proxy'];
if (environmentProxy) {
    needle.defaults({
        proxy: environmentProxy,
    });
}
exports.PACT_STANDALONE_VERSION = '3.1.2.2-alpha';
const PACT_DEFAULT_LOCATION = `https://github.com/you54f/pact-ruby-standalone/releases/download/v${exports.PACT_STANDALONE_VERSION}/`;
const HTTP_REGEX = /^http(s?):\/\//;
function join(...paths) {
    return HTTP_REGEX.test(paths[0]) ? urljoin(...paths) : path.join(...paths);
}
function throwError(msg) {
    throw new Error(chalk_1.default.red(`Error while installing binary: ${msg}`));
}
function makeError(msg) {
    return new Error(chalk_1.default.red(`Error while locating pact binary: ${msg}`));
}
function getBinaryLocation(location, basePath) {
    if (!location || location.length === 0) {
        return undefined;
    }
    return HTTP_REGEX.test(location)
        ? location
        : path.resolve(basePath, location);
}
function findPackageConfig(location, tries = 10) {
    if (tries === 0) {
        return {};
    }
    const packagePath = path.resolve(location, 'package.json');
    if (fs.existsSync(packagePath)) {
        const pkgConfig = require(packagePath).config;
        if (pkgConfig &&
            (pkgConfig.pact_binary_location || pkgConfig.pact_do_not_track)) {
            return {
                binaryLocation: getBinaryLocation(pkgConfig.pact_binary_location, location),
                doNotTrack: pkgConfig.pact_do_not_track,
            };
        }
    }
    return findPackageConfig(path.resolve(location, '..'), tries - 1);
}
function createConfig(location = process.cwd()) {
    const packageConfig = findPackageConfig(location);
    const PACT_BINARY_LOCATION = packageConfig.binaryLocation || PACT_DEFAULT_LOCATION;
    const CHECKSUM_SUFFIX = '.checksum';
    return {
        doNotTrack: packageConfig.doNotTrack ||
            process.env['PACT_DO_NOT_TRACK'] !== undefined ||
            false,
        binaries: [
            {
                platform: 'win32',
                binary: `pact-${exports.PACT_STANDALONE_VERSION}-windows-x86_64.zip`,
                binaryChecksum: `pact-${exports.PACT_STANDALONE_VERSION}-windows-x86_64.zip${CHECKSUM_SUFFIX}`,
                folderName: `windows-x64-${exports.PACT_STANDALONE_VERSION}`,
                downloadLocation: PACT_BINARY_LOCATION,
            },
            {
                platform: 'darwin',
                arch: 'x64',
                binary: `pact-${exports.PACT_STANDALONE_VERSION}-osx-x86_64.tar.gz`,
                binaryChecksum: `pact-${exports.PACT_STANDALONE_VERSION}-osx-x86_64.tar.gz${CHECKSUM_SUFFIX}`,
                folderName: `darwin-x64-${exports.PACT_STANDALONE_VERSION}`,
                downloadLocation: PACT_BINARY_LOCATION,
            },
            {
                platform: 'darwin',
                arch: 'arm64',
                binary: `pact-${exports.PACT_STANDALONE_VERSION}-osx-arm64.tar.gz`,
                binaryChecksum: `pact-${exports.PACT_STANDALONE_VERSION}-osx-arm64.tar.gz${CHECKSUM_SUFFIX}`,
                folderName: `darwin-arm64-${exports.PACT_STANDALONE_VERSION}`,
                downloadLocation: PACT_BINARY_LOCATION,
            },
            {
                platform: 'linux',
                arch: 'x64',
                binary: `pact-${exports.PACT_STANDALONE_VERSION}-linux-x86_64.tar.gz`,
                binaryChecksum: `pact-${exports.PACT_STANDALONE_VERSION}-linux-x86_64.tar.gz${CHECKSUM_SUFFIX}`,
                folderName: `linux-x64-${exports.PACT_STANDALONE_VERSION}`,
                downloadLocation: PACT_BINARY_LOCATION,
            },
            {
                platform: 'linux',
                arch: 'arm64',
                binary: `pact-${exports.PACT_STANDALONE_VERSION}-linux-arm64.tar.gz`,
                binaryChecksum: `pact-${exports.PACT_STANDALONE_VERSION}-linux-arm64.tar.gz${CHECKSUM_SUFFIX}`,
                folderName: `linux-arm64-${exports.PACT_STANDALONE_VERSION}`,
                downloadLocation: PACT_BINARY_LOCATION,
            },
        ],
    };
}
exports.createConfig = createConfig;
const CONFIG = createConfig();
const CIs = [
    'CI',
    'CONTINUOUS_INTEGRATION',
    'ABSTRUSE_BUILD_DIR',
    'APPVEYOR',
    'BUDDY_WORKSPACE_URL',
    'BUILDKITE',
    'CF_BUILD_URL',
    'CIRCLECI',
    'CODEBUILD_BUILD_ARN',
    'CONCOURSE_URL',
    'DRONE',
    'GITLAB_CI',
    'GO_SERVER_URL',
    'JENKINS_URL',
    'PROBO_ENVIRONMENT',
    'SEMAPHORE',
    'SHIPPABLE',
    'TDDIUM',
    'TEAMCITY_VERSION',
    'TF_BUILD',
    'TRAVIS',
    'WERCKER_ROOT',
];
function downloadFileRetry(url, filepath, retry = 3) {
    return new Promise((resolve, reject) => {
        let len = 0;
        let downloaded = 0;
        let time = Date.now();
        let ca = config.read().cafile;
        if (ca) {
            ca = fs.readFileSync(ca);
        }
        needle
            .get(url, {
            follow_max: 5,
            headers: {
                'User-Agent': 'https://github.com/pact-foundation/pact-node',
            },
            rejectUnauthorized: config.read()['strict-ssl'] || false,
            ca,
        })
            .on('error', (e) => reject(e))
            .on('response', (res) => (len = parseInt(res.headers['content-length'], 10)))
            .on('data', (chunk) => {
            downloaded += chunk.length;
            const now = Date.now();
            if (now - time > 1000) {
                time = now;
                console.log(chalk_1.default.gray(`Downloaded ${((100 * downloaded) / len).toFixed(2)}%...`));
            }
        })
            .pipe(fs.createWriteStream(filepath))
            .on('finish', resolve);
    }).catch((e) => retry-- === 0 ? throwError(e) : downloadFileRetry(url, filepath, retry));
}
function download(data) {
    console.log(chalk_1.default.gray(`Installing Pact Standalone Binary for ${data.platform}.`));
    return new Promise((resolve, reject) => {
        if (fs.existsSync(path.resolve(data.filepath))) {
            console.log(chalk_1.default.yellow('Binary already downloaded, skipping...'));
            resolve({ ...data, binaryAlreadyDownloaded: true });
            return;
        }
        console.log(chalk_1.default.yellow(`Downloading Pact Standalone Binary v${exports.PACT_STANDALONE_VERSION} for platform ${data.platform} from ${data.binaryDownloadPath}`));
        if (!CONFIG.doNotTrack) {
            console.log(chalk_1.default.gray('Please note: we are tracking this download anonymously to gather important usage statistics. ' +
                "To disable tracking, set 'pact_do_not_track: true' in your package.json 'config' section."));
            const isCI = CIs.some((key) => process.env[key] !== undefined);
            needle
                .post('https://www.google-analytics.com/collect', {
                form: {
                    v: 1,
                    tid: 'UA-117778936-1',
                    cid: Math.round(2147483647 * Math.random()).toString(),
                    t: 'screenview',
                    an: 'pact-install',
                    av: require('../package.json').version,
                    aid: 'pact-node',
                    aiid: `standalone-${exports.PACT_STANDALONE_VERSION}`,
                    cd: `download-node-${data.platform}-${isCI ? 'ci' : 'user'}`,
                    aip: true,
                },
            })
                .on('error', () => {
            });
        }
        if (HTTP_REGEX.test(data.binaryDownloadPath)) {
            downloadFileRetry(data.binaryDownloadPath, data.filepath).then(() => {
                console.log(chalk_1.default.green(`Finished downloading binary to ${data.filepath}`));
                resolve(data);
            }, (e) => reject(`Error downloading binary from ${data.binaryDownloadPath}: ${e}`));
        }
        else if (fs.existsSync(data.binaryDownloadPath)) {
            fs.createReadStream(data.binaryDownloadPath)
                .on('error', (e) => reject(`Error reading the file at '${data.binaryDownloadPath}': ${e}`))
                .pipe(fs
                .createWriteStream(data.filepath)
                .on('error', (e) => reject(`Error writing the file to '${data.filepath}': ${e}`))
                .on('close', () => resolve(data)));
        }
        else {
            reject(`Could not get binary from '${data.binaryDownloadPath}' as it's not a URL and does not exist at the path specified.`);
        }
    });
}
function extract(data) {
    if (fs.existsSync(data.platformFolderPath)) {
        return Promise.resolve({ ...data, binaryAlreadyInstalled: true });
    }
    if (!fs.existsSync(data.checksumFilepath)) {
        throwError(`Checksum file missing from standalone directory. Aborting.`);
    }
    fs.mkdirSync(data.platformFolderPath);
    console.log(chalk_1.default.yellow(`Extracting binary from ${data.filepath}.`));
    const basename = path.basename(data.filepath);
    return (sumchecker('sha1', data.checksumFilepath, __dirname, basename)
        .then(() => console.log(chalk_1.default.green(`Checksum passed for '${basename}'.`)), () => throwError(`Checksum rejected for file '${basename}' with checksum ${path.basename(data.checksumFilepath)}`))
        .then(() => data.isWindows
        ? fs
            .createReadStream(data.filepath)
            .pipe(unzipper.Extract({
            path: data.platformFolderPath,
        }))
            .on('entry', (entry) => entry.autodrain())
            .promise()
        : tar.x({
            file: data.filepath,
            cwd: data.platformFolderPath,
            preserveOwner: false,
        }))
        .then(() => {
        const publishPath = path.resolve(data.platformFolderPath, 'pact', 'bin', `pact-publish${pactEnvironment.isWindows() ? '.bat' : ''}`);
        if (fs.existsSync(publishPath)) {
            fs.unlinkSync(publishPath);
        }
        console.log(chalk_1.default.green('Extraction done.'));
    })
        .then(() => {
        console.log(`\n\n${chalk_1.default.bgYellow(chalk_1.default.black('### If you') +
            chalk_1.default.red(' ❤ ') +
            chalk_1.default.black('Pact and want to support us, please donate here:'))}${chalk_1.default.blue(' http://donate.pact.io/node')}\n\n`);
        return Promise.resolve(data);
    })
        .catch((e) => throwError(`Extraction failed for ${data.filepath}: ${e}`)));
}
function getBinaryEntry(platform = process.platform, arch = process.arch) {
    const found = CONFIG.binaries.find((value) => value.platform === platform && (value.arch ? value.arch === arch : true));
    if (found === undefined) {
        throw makeError(`Cannot find binary for platform '${platform}' with architecture '${arch}'.`);
    }
    return found;
}
exports.getBinaryEntry = getBinaryEntry;
function setup(platform, arch) {
    const entry = getBinaryEntry(platform, arch);
    return Promise.resolve({
        binaryDownloadPath: join(entry.downloadLocation, entry.binary),
        checksumDownloadPath: join(PACT_DEFAULT_LOCATION, entry.binaryChecksum),
        filepath: path.resolve(__dirname, entry.binary),
        checksumFilepath: path.resolve(__dirname, entry.binaryChecksum),
        isWindows: pactEnvironment.isWindows(platform),
        platform: entry.platform,
        arch: entry.arch,
        platformFolderPath: path.resolve(__dirname, entry.folderName),
    });
}
function downloadChecksums() {
    console.log(chalk_1.default.gray(`Downloading All Pact Standalone Binary Checksums.`));
    return Promise.all(CONFIG.binaries.map((value) => setup(value.platform, value.arch).then((data) => downloadFileRetry(data.checksumDownloadPath, data.checksumFilepath).then(() => {
        console.log(chalk_1.default.green(`Finished downloading checksum ${path.basename(data.checksumFilepath)}`));
        return data;
    }, (e) => throwError(`Error downloading checksum from ${data.checksumDownloadPath}: ${e}`))))).then(() => console.log(chalk_1.default.green('All checksums downloaded.')), (e) => throwError(`Checksum Download Failed Unexpectedly: ${e}`));
}
exports.downloadChecksums = downloadChecksums;
exports.default = (platform, arch) => {
    if (process.env['PACT_SKIP_BINARY_INSTALL'] === 'true') {
        console.log(chalk_1.default.yellow("Skipping binary installation. Env var 'PACT_SKIP_BINARY_INSTALL' was found."));
        return Promise.resolve({
            binaryInstallSkipped: true,
        });
    }
    return setup(platform, arch)
        .then(download)
        .then(extract)
        .then((d) => {
        console.log(chalk_1.default.green('Pact Standalone Binary is ready.'));
        return { ...d, binaryInstallSkipped: false };
    })
        .catch((e) => throwError(`Postinstalled Failed Unexpectedly: ${e}`));
};
//# sourceMappingURL=install.js.map